package com.tearabite.hardware;

import com.pi4j.context.Context;
import com.pi4j.io.gpio.digital.DigitalOutput;
import com.pi4j.io.gpio.digital.DigitalState;
import com.pi4j.util.Console;
import com.tearabite.base.I2CDevice;

import java.nio.ByteBuffer;

import static java.lang.Thread.sleep;

public class BoardCore extends I2CDevice implements Runnable {

    private static final int I2C_BAT_ADDR = 0;
    private static final byte I2C_SERVO_ADDR = 21;
    private static final int I2C_SERVO_CMD_ADDR = 40;
    private static final int I2C_MOTOR_ADDR = 31;
    private static final int I2C_BUS = 1;
    private static final int I2C_DEVICE_ADDR = 0x7A;
    private static final int BUZZER_PIN = 6;

    private static final int LOBOT_SERVO_FRAME_HEADER = 0x55;
    private static final int LOBOT_SERVO_MOVE_TIME_WRITE = 1;
    private static final int LOBOT_SERVO_MOVE_TIME_READ = 2;
    private static final int LOBOT_SERVO_MOVE_TIME_WAIT_WRITE = 7;
    private static final int LOBOT_SERVO_MOVE_TIME_WAIT_READ = 8;
    private static final int LOBOT_SERVO_MOVE_START = 11;
    private static final int LOBOT_SERVO_MOVE_STOP = 12;
    private static final int LOBOT_SERVO_ID_WRITE = 13;
    private static final int LOBOT_SERVO_ID_READ = 14;
    private static final int LOBOT_SERVO_ANGLE_OFFSET_ADJUST = 17;
    private static final int LOBOT_SERVO_ANGLE_OFFSET_WRITE = 18;
    private static final int LOBOT_SERVO_ANGLE_OFFSET_READ = 19;
    private static final int LOBOT_SERVO_ANGLE_LIMIT_WRITE = 20;
    private static final int LOBOT_SERVO_ANGLE_LIMIT_READ = 21;
    private static final int LOBOT_SERVO_VIN_LIMIT_WRITE = 22;
    private static final int LOBOT_SERVO_VIN_LIMIT_READ = 23;
    private static final int LOBOT_SERVO_TEMP_MAX_LIMIT_WRITE = 24;
    private static final int LOBOT_SERVO_TEMP_MAX_LIMIT_READ = 25;
    private static final int LOBOT_SERVO_TEMP_READ = 26;
    private static final int LOBOT_SERVO_VIN_READ = 27;
    private static final int LOBOT_SERVO_POS_READ = 28;
    private static final int LOBOT_SERVO_OR_MOTOR_MODE_WRITE = 29;
    private static final int LOBOT_SERVO_OR_MOTOR_MODE_READ = 30;
    private static final int LOBOT_SERVO_LOAD_OR_UNLOAD_WRITE = 31;
    private static final int LOBOT_SERVO_LOAD_OR_UNLOAD_READ = 32;
    private static final int LOBOT_SERVO_LED_CTRL_WRITE = 33;
    private static final int LOBOT_SERVO_LED_CTRL_READ = 34;
    private static final int LOBOT_SERVO_LED_ERROR_WRITE = 35;
    private static final int LOBOT_SERVO_LED_ERROR_READ = 36;

//            GPIO.setwarnings(False)
//            GPIO.setmode(GPIO.BOARD)

    private static final int __RGB_COUNT = 2;
    private static final int __RGB_PIN = 12;
    private static final int __RGB_FREQ_HZ = 800000;
    private static final int __RGB_DMA = 10;
    private static final int __RGB_BRIGHTNESS = 120;
    private static final int __RGB_CHANNEL = 0;
    private static final boolean __RGB_INVERT = false;
    private static final Console console = new Console();
    private final DigitalOutput buzzer;

    protected final int[] motorSpeeds = new int[] { 0, 0, 0, 0 };
    protected final int[] servoAngles = new int[] { 0, 0, 0, 0, 0, 0 };
    protected int battery;
    protected long beepUntil;

    public BoardCore(Context pi4j) {
        super(pi4j, I2C_DEVICE_ADDR, "MasterPiBoard");
        this.buzzer = pi4j.create(DigitalOutput.newConfigBuilder(pi4j)
                .address(BUZZER_PIN)
                .shutdown(DigitalState.LOW)
                .initial(DigitalState.LOW)
                .provider("pigpio-digital-output")
                .build());
    }

    private void setMotor(int index, int speed) {
        if (!i2c.isOpen()) {
            return;
        }

        byte reg = (byte) (I2C_MOTOR_ADDR + index);
        i2c.writeRegister(reg, speed);
    }

    private void setServo(int index, int angle) {
        if (!i2c.isOpen()) {
            return;
        }

        byte reg = (byte) (I2C_SERVO_ADDR + index);
        i2c.writeRegister(reg, angle);
    }

    private void updateBattery() {
        i2c.write(I2C_BAT_ADDR);
        byte[] bytes = i2c.readNBytes(2);
        this.battery = ByteBuffer.wrap(bytes).getShort();
    }

    private void startBeep() {
        buzzer.high();
    }

    private void stopBeep() {
        buzzer.low();
    }

    private void scan() {
        for (int motorIndex = 0; motorIndex < 4; motorIndex++) {
            this.setMotor(motorIndex, motorSpeeds[motorIndex]);
        }

        for (int servoIndex = 0; servoIndex < 6; servoIndex++) {
            this.setServo(servoIndex, servoAngles[servoIndex]);
        }

        this.updateBattery();

        if (System.currentTimeMillis() < this.beepUntil) {
            this.startBeep();
        } else {
            this.stopBeep();
        }
    }

    public void start() {
        new Thread(this).start();
    }

    @Override
    public void run() {
        try {
            while (true) {
                scan();
                sleep(20);
            }
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        }
    }
}

